@@ -16,10 +16,9 @@
 // under the License.
 
 //! Parquet integration tests
+use crate::parquet::utils::MetricsFinder;
 use arrow::array::Decimal128Array;
-use arrow::datatypes::{
-    i256, IntervalDayTimeType, IntervalMonthDayNanoType, IntervalYearMonthType,
-};
+use arrow::datatypes::i256;
 use arrow::{
     array::{
         make_array, Array, ArrayRef, BinaryArray, BooleanArray, Date32Array, Date64Array,
@@ -35,24 +34,24 @@ use arrow::{
     record_batch::RecordBatch,
     util::pretty::pretty_format_batches,
 };
-use arrow_array::{
-    IntervalDayTimeArray, IntervalMonthDayNanoArray, IntervalYearMonthArray,
-};
-use arrow_schema::IntervalUnit;
 use chrono::{Datelike, Duration, TimeDelta};
 use datafusion::{
-    datasource::{physical_plan::ParquetExec, provider_as_source, TableProvider},
-    physical_plan::{accept, metrics::MetricsSet, ExecutionPlan, ExecutionPlanVisitor},
+    datasource::{provider_as_source, TableProvider},
+    physical_plan::metrics::MetricsSet,
     prelude::{ParquetReadOptions, SessionConfig, SessionContext},
 };
 use datafusion_expr::{Expr, LogicalPlan, LogicalPlanBuilder};
 use half::f16;
 use parquet::arrow::ArrowWriter;
-use parquet::file::properties::WriterProperties;
+use parquet::file::properties::{EnabledStatistics, WriterProperties};
 use std::sync::Arc;
 use tempfile::NamedTempFile;
+
 mod arrow_statistics;
 mod custom_reader;
+// Don't run on windows as tempfiles don't seem to work the same
+#[cfg(not(target_os = "windows"))]
+mod external_access_plan;
 mod file_statistics;
 #[cfg(not(target_family = "windows"))]
 mod filter_pushdown;
@@ -60,6 +59,7 @@ mod page_pruning;
 mod row_group_pruning;
 mod schema;
 mod schema_coercion;
+mod utils;
 
 #[cfg(test)]
 #[ctor::ctor]
@@ -86,11 +86,11 @@ enum Scenario {
     Time32Millisecond,
     Time64Nanosecond,
     Time64Microsecond,
-    Interval,
     /// 7 Rows, for each i8, i16, i32, i64, u8, u16, u32, u64, f32, f64
     /// -MIN, -100, -1, 0, 1, 100, MAX
     NumericLimits,
     Float16,
+    Float32,
     Float64,
     Decimal,
     Decimal256,
@@ -303,25 +303,8 @@ impl ContextWithParquet {
             .expect("Running");
 
         // find the parquet metrics
-        struct MetricsFinder {
-            metrics: Option<MetricsSet>,
-        }
-        impl ExecutionPlanVisitor for MetricsFinder {
-            type Error = std::convert::Infallible;
-            fn pre_visit(
-                &mut self,
-                plan: &dyn ExecutionPlan,
-            ) -> Result<bool, Self::Error> {
-                if plan.as_any().downcast_ref::<ParquetExec>().is_some() {
-                    self.metrics = plan.metrics();
-                }
-                // stop searching once we have found the metrics
-                Ok(self.metrics.is_none())
-            }
-        }
-        let mut finder = MetricsFinder { metrics: None };
-        accept(physical_plan.as_ref(), &mut finder).unwrap();
-        let parquet_metrics = finder.metrics.unwrap();
+        let parquet_metrics =
+            MetricsFinder::find_metrics(physical_plan.as_ref()).unwrap();
 
         let result_rows = results.iter().map(|b| b.num_rows()).sum();
 
@@ -604,6 +587,12 @@ fn make_f64_batch(v: Vec<f64>) -> RecordBatch {
     RecordBatch::try_new(schema, vec![array.clone()]).unwrap()
 }
 
+fn make_f32_batch(v: Vec<f32>) -> RecordBatch {
+    let schema = Arc::new(Schema::new(vec![Field::new("f", DataType::Float32, true)]));
+    let array = Arc::new(Float32Array::from(v)) as ArrayRef;
+    RecordBatch::try_new(schema, vec![array.clone()]).unwrap()
+}
+
 fn make_f16_batch(v: Vec<f16>) -> RecordBatch {
     let schema = Arc::new(Schema::new(vec![Field::new("f", DataType::Float16, true)]));
     let array = Arc::new(Float16Array::from(v)) as ArrayRef;
@@ -932,71 +921,6 @@ fn make_dict_batch() -> RecordBatch {
     .unwrap()
 }
 
-fn make_interval_batch(offset: i32) -> RecordBatch {
-    let schema = Schema::new(vec![
-        Field::new(
-            "year_month",
-            DataType::Interval(IntervalUnit::YearMonth),
-            true,
-        ),
-        Field::new("day_time", DataType::Interval(IntervalUnit::DayTime), true),
-        Field::new(
-            "month_day_nano",
-            DataType::Interval(IntervalUnit::MonthDayNano),
-            true,
-        ),
-    ]);
-    let schema = Arc::new(schema);
-
-    let ym_arr = IntervalYearMonthArray::from(vec![
-        Some(IntervalYearMonthType::make_value(1 + offset, 10 + offset)),
-        Some(IntervalYearMonthType::make_value(2 + offset, 20 + offset)),
-        Some(IntervalYearMonthType::make_value(3 + offset, 30 + offset)),
-        None,
-        Some(IntervalYearMonthType::make_value(5 + offset, 50 + offset)),
-    ]);
-
-    let dt_arr = IntervalDayTimeArray::from(vec![
-        Some(IntervalDayTimeType::make_value(1 + offset, 10 + offset)),
-        Some(IntervalDayTimeType::make_value(2 + offset, 20 + offset)),
-        Some(IntervalDayTimeType::make_value(3 + offset, 30 + offset)),
-        None,
-        Some(IntervalDayTimeType::make_value(5 + offset, 50 + offset)),
-    ]);
-
-    // Not yet implemented, refer to:
-    // https://github.com/apache/arrow-rs/blob/master/parquet/src/arrow/arrow_writer/mod.rs#L747
-    let mdn_arr = IntervalMonthDayNanoArray::from(vec![
-        Some(IntervalMonthDayNanoType::make_value(
-            1 + offset,
-            10 + offset,
-            100 + (offset as i64),
-        )),
-        Some(IntervalMonthDayNanoType::make_value(
-            2 + offset,
-            20 + offset,
-            200 + (offset as i64),
-        )),
-        Some(IntervalMonthDayNanoType::make_value(
-            3 + offset,
-            30 + offset,
-            300 + (offset as i64),
-        )),
-        None,
-        Some(IntervalMonthDayNanoType::make_value(
-            5 + offset,
-            50 + offset,
-            500 + (offset as i64),
-        )),
-    ]);
-
-    RecordBatch::try_new(
-        schema,
-        vec![Arc::new(ym_arr), Arc::new(dt_arr), Arc::new(mdn_arr)],
-    )
-    .unwrap()
-}
-
 fn create_data_batch(scenario: Scenario) -> Vec<RecordBatch> {
     match scenario {
         Scenario::Boolean => {
@@ -1086,6 +1010,14 @@ fn create_data_batch(scenario: Scenario) -> Vec<RecordBatch> {
                 ),
             ]
         }
+        Scenario::Float32 => {
+            vec![
+                make_f32_batch(vec![-5.0, -4.0, -3.0, -2.0, -1.0]),
+                make_f32_batch(vec![-4.0, -3.0, -2.0, -1.0, 0.0]),
+                make_f32_batch(vec![0.0, 1.0, 2.0, 3.0, 4.0]),
+                make_f32_batch(vec![5.0, 6.0, 7.0, 8.0, 9.0]),
+            ]
+        }
         Scenario::Float64 => {
             vec![
                 make_f64_batch(vec![-5.0, -4.0, -3.0, -2.0, -1.0]),
@@ -1418,12 +1350,6 @@ fn create_data_batch(scenario: Scenario) -> Vec<RecordBatch> {
                 ]),
             ]
         }
-        Scenario::Interval => vec![
-            make_interval_batch(0),
-            make_interval_batch(1),
-            make_interval_batch(2),
-            make_interval_batch(3),
-        ],
     }
 }
 
@@ -1438,6 +1364,7 @@ async fn make_test_file_rg(scenario: Scenario, row_per_group: usize) -> NamedTem
     let props = WriterProperties::builder()
         .set_max_row_group_size(row_per_group)
         .set_bloom_filter_enabled(true)
+        .set_statistics_enabled(EnabledStatistics::Page)
         .build();
 
     let batches = create_data_batch(scenario);
