@@ -17,18 +17,21 @@
 
 use std::sync::Arc;
 
+use arrow::array::{make_array, Array, ArrayRef, BooleanArray, MutableArrayData};
+use arrow::compute::{and_kleene, is_not_null, SlicesIterator};
+use arrow::datatypes::Schema;
+
+use datafusion_common::{exec_err, Result};
+use datafusion_expr::expr::Alias;
+use datafusion_expr::sort_properties::ExprProperties;
+use datafusion_expr::Expr;
+
+use crate::expressions::literal::Literal;
 use crate::expressions::{self, CastExpr};
 use crate::physical_expr::PhysicalExpr;
 use crate::sort_expr::PhysicalSortExpr;
 use crate::tree_node::ExprContext;
 
-use arrow::array::{make_array, Array, ArrayRef, BooleanArray, MutableArrayData};
-use arrow::compute::{and_kleene, is_not_null, SlicesIterator};
-use arrow::datatypes::Schema;
-use datafusion_common::{exec_err, Result};
-use datafusion_expr::sort_properties::ExprProperties;
-use datafusion_expr::Expr;
-
 /// Represents a [`PhysicalExpr`] node with associated properties (order and
 /// range) in a context where properties are tracked.
 pub type ExprPropertiesNode = ExprContext<ExprProperties>;
@@ -101,10 +104,7 @@ pub fn scatter(mask: &BooleanArray, truthy: &dyn Array) -> Result<ArrayRef> {
 pub fn reverse_order_bys(order_bys: &[PhysicalSortExpr]) -> Vec<PhysicalSortExpr> {
     order_bys
         .iter()
-        .map(|e| PhysicalSortExpr {
-            expr: e.expr.clone(),
-            options: !e.options,
-        })
+        .map(|e| PhysicalSortExpr::new(e.expr.clone(), !e.options))
         .collect()
 }
 
@@ -115,6 +115,9 @@ pub fn limited_convert_logical_expr_to_physical_expr(
     schema: &Schema,
 ) -> Result<Arc<dyn PhysicalExpr>> {
     match expr {
+        Expr::Alias(Alias { expr, .. }) => {
+            Ok(limited_convert_logical_expr_to_physical_expr(expr, schema)?)
+        }
         Expr::Column(col) => expressions::column::col(&col.name, schema),
         Expr::Cast(cast_expr) => Ok(Arc::new(CastExpr::new(
             limited_convert_logical_expr_to_physical_expr(
@@ -124,10 +127,7 @@ pub fn limited_convert_logical_expr_to_physical_expr(
             cast_expr.data_type.clone(),
             None,
         ))),
-        Expr::Alias(alias_expr) => limited_convert_logical_expr_to_physical_expr(
-            alias_expr.expr.as_ref(),
-            schema,
-        ),
+        Expr::Literal(value) => Ok(Arc::new(Literal::new(value.clone()))),
         _ => exec_err!(
             "Unsupported expression: {expr} for conversion to Arc<dyn PhysicalExpr>"
         ),
@@ -138,11 +138,12 @@ pub fn limited_convert_logical_expr_to_physical_expr(
 mod tests {
     use std::sync::Arc;
 
-    use super::*;
-
     use arrow::array::Int32Array;
+
     use datafusion_common::cast::{as_boolean_array, as_int32_array};
 
+    use super::*;
+
     #[test]
     fn scatter_int() -> Result<()> {
         let truthy = Arc::new(Int32Array::from(vec![1, 10, 11, 100]));
