@@ -19,15 +19,13 @@ mod adapter;
 pub use adapter::GroupsAccumulatorAdapter;
 
 // Backward compatibility
+#[allow(unused_imports)]
 pub(crate) mod accumulate {
-    pub use datafusion_physical_expr_common::aggregate::groups_accumulator::accumulate::{accumulate_indices, NullState};
+    pub use datafusion_physical_expr_common::aggregate::groups_accumulator::accumulate::NullState;
 }
 
 pub use datafusion_physical_expr_common::aggregate::groups_accumulator::accumulate::NullState;
 
-pub(crate) mod bool_op {
-    pub use datafusion_physical_expr_common::aggregate::groups_accumulator::bool_op::BooleanGroupsAccumulator;
-}
 pub(crate) mod prim_op {
     pub use datafusion_physical_expr_common::aggregate::groups_accumulator::prim_op::PrimitiveGroupsAccumulator;
 }
