@@ -17,35 +17,13 @@
 
 pub use datafusion_physical_expr_common::aggregate::AggregateExpr;
 
-mod hyperloglog;
-mod tdigest;
-
-pub(crate) mod approx_distinct;
-pub(crate) mod approx_median;
-pub(crate) mod approx_percentile_cont;
-pub(crate) mod approx_percentile_cont_with_weight;
 pub(crate) mod array_agg;
 pub(crate) mod array_agg_distinct;
 pub(crate) mod array_agg_ordered;
-pub(crate) mod average;
-pub(crate) mod bit_and_or_xor;
-pub(crate) mod bool_and_or;
-pub(crate) mod correlation;
-pub(crate) mod count;
-pub(crate) mod count_distinct;
-pub(crate) mod covariance;
-pub(crate) mod grouping;
-pub(crate) mod nth_value;
-pub(crate) mod string_agg;
 #[macro_use]
 pub(crate) mod min_max;
 pub(crate) mod groups_accumulator;
-pub(crate) mod regr;
 pub(crate) mod stats;
-pub(crate) mod stddev;
-pub(crate) mod sum;
-pub(crate) mod sum_distinct;
-pub(crate) mod variance;
 
 pub mod build_in;
 pub mod moving_min_max;
